@@ -1,5 +1,6 @@
 /*
   SPDX-FileCopyrightText: 2002 Marc Mutz <mutz@kde.org>
+  SPDX-FileCopyrightText: 2024 Laurent Montel <montel@kde.org>
 
   SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -17,6 +18,8 @@
 #include "identitycombo.h"
 #include "identity.h"
 #include "identitymanager.h"
+#include "identitytreemodel.h"
+#include "identitytreesortproxymodel.h"
 
 #include <KLocalizedString>
 
@@ -33,79 +36,62 @@ class KIdentityManagementWidgets::IdentityComboPrivate
 {
 public:
     IdentityComboPrivate(KIdentityManagementCore::IdentityManager *manager, IdentityCombo *qq)
-        : mIdentityManager(manager)
+        : mIdentityModel(new KIdentityManagementCore::IdentityTreeModel(manager, qq))
+        , mIdentityProxyModel(new KIdentityManagementCore::IdentityTreeSortProxyModel(qq))
         , q(qq)
     {
+        mIdentityProxyModel->setSourceModel(mIdentityModel);
+        q->setModel(mIdentityProxyModel);
     }
 
-    void reloadCombo();
-    void reloadUoidList();
-
-    QList<uint> mUoidList;
-    KIdentityManagementCore::IdentityManager *const mIdentityManager;
+    KIdentityManagementCore::IdentityTreeModel *const mIdentityModel;
+    KIdentityManagementCore::IdentityTreeSortProxyModel *const mIdentityProxyModel;
     IdentityCombo *const q;
-    bool showDefault = false;
 };
 
-void KIdentityManagementWidgets::IdentityComboPrivate::reloadCombo()
-{
-    QStringList identities;
-    identities.reserve(mIdentityManager->identities().count());
-    KIdentityManagementCore::IdentityManager::ConstIterator it;
-    KIdentityManagementCore::IdentityManager::ConstIterator end(mIdentityManager->end());
-    for (it = mIdentityManager->begin(); it != end; ++it) {
-        if (showDefault && it->isDefault()) {
-            identities << QString(it->identityName() + i18nc("Default identity", " (default)"));
-        } else {
-            identities << it->identityName();
-        }
-    }
-    // the IM should prevent this from happening:
-    assert(!identities.isEmpty());
-    q->clear();
-    q->addItems(identities);
-}
-
-void KIdentityManagementWidgets::IdentityComboPrivate::reloadUoidList()
-{
-    mUoidList.clear();
-    KIdentityManagementCore::IdentityManager::ConstIterator it;
-    KIdentityManagementCore::IdentityManager::ConstIterator end(mIdentityManager->end());
-    for (it = mIdentityManager->begin(); it != end; ++it) {
-        mUoidList << (*it).uoid();
-    }
-}
-
 //@endcond
 
 IdentityCombo::IdentityCombo(IdentityManager *manager, QWidget *parent)
     : QComboBox(parent)
     , d(new KIdentityManagementWidgets::IdentityComboPrivate(manager, this))
 {
-    d->reloadCombo();
-    d->reloadUoidList();
-    connect(this, &IdentityCombo::activated, this, &IdentityCombo::slotEmitChanged);
-    connect(this, &IdentityCombo::identityChanged, this, &IdentityCombo::slotUpdateTooltip);
     connect(manager, &KIdentityManagementCore::IdentityManager::identitiesWereChanged, this, &IdentityCombo::slotIdentityManagerChanged);
     connect(manager, &KIdentityManagementCore::IdentityManager::deleted, this, &IdentityCombo::identityDeleted);
+    connect(this, &IdentityCombo::activated, this, &IdentityCombo::slotEmitChanged);
+    connect(this, &IdentityCombo::identityChanged, this, &IdentityCombo::slotUpdateTooltip);
+    setModelColumn(KIdentityManagementCore::IdentityTreeModel::DisplayIdentityNameRole);
     slotUpdateTooltip(currentIdentity());
 }
 
 IdentityCombo::~IdentityCombo() = default;
 
+IdentityActivitiesAbstract *IdentityCombo::identityActivitiesAbstract() const
+{
+    return d->mIdentityProxyModel->identityActivitiesAbstract();
+}
+
+void IdentityCombo::setIdentityActivitiesAbstract(IdentityActivitiesAbstract *newIdentityActivitiesAbstract)
+{
+    d->mIdentityProxyModel->setIdentityActivitiesAbstract(newIdentityActivitiesAbstract);
+}
+
 QString IdentityCombo::currentIdentityName() const
 {
-    return d->mIdentityManager->identities().at(currentIndex());
+    return d->mIdentityProxyModel->mapToSource(d->mIdentityProxyModel->index(currentIndex(), KIdentityManagementCore::IdentityTreeModel::IdentityNameRole))
+        .data()
+        .toString();
 }
 
 uint IdentityCombo::currentIdentity() const
 {
-    return d->mUoidList.at(currentIndex());
+    return d->mIdentityProxyModel->mapToSource(d->mIdentityProxyModel->index(currentIndex(), KIdentityManagementCore::IdentityTreeModel::UoidRole))
+        .data()
+        .toInt();
 }
 
 bool IdentityCombo::isDefaultIdentity() const
 {
-    return d->mUoidList.at(currentIndex()) == d->mIdentityManager->defaultIdentity().uoid();
+    return currentIdentity() == d->mIdentityModel->identityManager()->defaultIdentity().uoid();
 }
 
 void IdentityCombo::setCurrentIdentity(const Identity &identity)
@@ -118,21 +104,24 @@ void IdentityCombo::setCurrentIdentity(const QString &name)
     if (name.isEmpty()) {
         return;
     }
-    const int idx = d->mIdentityManager->identities().indexOf(name);
-    if (idx < 0) {
+
+    const int idx = d->mIdentityModel->identityManager()->identities().indexOf(name);
+    const int newIndex = d->mIdentityProxyModel->mapFromSource(d->mIdentityModel->index(idx, KIdentityManagementCore::IdentityTreeModel::UoidRole)).row();
+
+    if (newIndex < 0) {
         Q_EMIT invalidIdentity();
         return;
     }
 
-    if (idx == currentIndex()) {
+    if (newIndex == currentIndex()) {
         return;
     }
 
     blockSignals(true); // just in case Qt gets fixed to emit activated() here
-    setCurrentIndex(idx);
+    setCurrentIndex(newIndex);
     blockSignals(false);
 
-    slotEmitChanged(idx);
+    slotEmitChanged(newIndex);
 }
 
 void IdentityCombo::setCurrentIdentity(uint uoid)
@@ -140,37 +129,38 @@ void IdentityCombo::setCurrentIdentity(uint uoid)
     if (uoid == 0) {
         return;
     }
-    const int idx = d->mUoidList.indexOf(uoid);
-    if (idx < 0) {
+    const int idx = d->mIdentityModel->uoidIndex(uoid);
+    const int newIndex = d->mIdentityProxyModel->mapFromSource(d->mIdentityModel->index(idx, KIdentityManagementCore::IdentityTreeModel::UoidRole)).row();
+
+    if (newIndex < 0) {
         Q_EMIT invalidIdentity();
         return;
     }
-    if (idx == currentIndex()) {
+    if (newIndex == currentIndex()) {
         return;
     }
 
     blockSignals(true); // just in case Qt gets fixed to emit activated() here
-    setCurrentIndex(idx);
+    setCurrentIndex(newIndex);
     blockSignals(false);
 
-    slotEmitChanged(idx);
+    slotEmitChanged(newIndex);
 }
 
 void IdentityCombo::slotIdentityManagerChanged()
 {
-    uint oldIdentity = d->mUoidList.at(currentIndex());
+    const uint oldIdentity = currentIdentity();
 
-    d->reloadUoidList();
-    int idx = d->mUoidList.indexOf(oldIdentity);
+    const int idx = d->mIdentityModel->uoidIndex(oldIdentity);
+    const int newIndex = d->mIdentityProxyModel->mapFromSource(d->mIdentityModel->index(idx, KIdentityManagementCore::IdentityTreeModel::UoidRole)).row();
 
     blockSignals(true);
-    d->reloadCombo();
-    setCurrentIndex(idx < 0 ? 0 : idx);
+    setCurrentIndex(newIndex < 0 ? 0 : newIndex);
     blockSignals(false);
 
     slotUpdateTooltip(currentIdentity());
 
-    if (idx < 0) {
+    if (newIndex < 0) {
         // apparently our oldIdentity got deleted:
         slotEmitChanged(currentIndex());
     }
@@ -178,25 +168,22 @@ void IdentityCombo::slotIdentityManagerChanged()
 
 void IdentityCombo::slotEmitChanged(int idx)
 {
-    Q_EMIT identityChanged(d->mUoidList.at(idx));
+    Q_EMIT identityChanged(d->mIdentityModel->identityUoid(idx));
 }
 
 void IdentityCombo::slotUpdateTooltip(uint uoid)
 {
-    setToolTip(d->mIdentityManager->identityForUoid(uoid).fullEmailAddr());
+    setToolTip(d->mIdentityModel->identityManager()->identityForUoid(uoid).fullEmailAddr());
 }
 
 IdentityManager *IdentityCombo::identityManager() const
 {
-    return d->mIdentityManager;
+    return d->mIdentityModel->identityManager();
 }
 
 void IdentityCombo::setShowDefault(bool showDefault)
 {
-    if (d->showDefault != showDefault) {
-        d->showDefault = showDefault;
-        d->reloadCombo();
-    }
+    d->mIdentityModel->setShowDefault(showDefault);
 }
 
 #include "moc_identitycombo.cpp"
